#!/bin/ksh
# ----------------------------------------------------------------------------
# Copyright (C) 2012  Rodrigo de Araujo Jorge
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------
# Written by Rodrigo Jorge <http://www.dbarj.com.br/>
# Last updated on: Oct/18/2012 by Rodrigo Jorge
# ----------------------------------------------------------------------------

###############################################
###############################################
#########                             #########
#########           WARNING           #########
#########   DO NOT TOUCH THIS FILE    #########
#########                             #########
###############################################
###############################################

###############################################
###############################################
####                                      #####
####     ANY MODIFICATION MUST BE DONE    #####
####     INSIDE "oracle_sid.cfg" FILE     #####
####                                      #####
###############################################
###############################################

# Function to print in log file.
imprime ()
{
	echo "$(date "+%F %T") - $1"
}

# Function to exit this script.
exitscript ()
{
	if [ "$PWDFILE" != "" ]
	then
		test -f $PWDFILE && (imprime "Removing password file..."; rm -f $PWDFILE)
	fi
	if [ "$EXPPIPE" != "" ]
	then
		test -p $EXPPIPE && (imprime "Removing pipe..."; rm -f $EXPPIPE)
	fi
	exit $1
}

# Function to verify if bin exists.
bincheck ()
{
	run=$(which $1)
	ret=$?

	if [ $ret -ne 0 -o "$(echo $run | $HEADCMD 1)" != "/" ]
	then
		imprime "The \"$1\" command could not be found. Please add to PATH..."
		exitscript 1
	fi
}

# Function to verify if variable is declared.
varcheck ()
{
	run=$(eval "echo \"\$$1\"")
	if [ "$run" = "" ]
	then
		imprime "Variable \"$1\" not defined in parameter file..."
		exitscript 1
	fi
}

# Function to return if file is being used.
fileusedchk ()
{
	# 1 - Being used
	# 0 - Unused
	
	if [ "$SOTYPE" = "HP-UX" ]
	then
		run=$(fuser $1 2>&- | wc -l)
	elif [ "$SOTYPE" = "Linux" ]
	then
		run=$(ls -la /proc/*/fd/* 2>&- | grep $1 | wc -l)
	elif [ "$SOTYPE" = "SunOS" ]
	then
		run=$(fuser $1 2>&- | wc -w)
	fi

	if [ "$run" = "1" ]
	then
		return 1
	else
		return 0
	fi
}

# Function to run with trap.
abortexp ()
{
	imprime "Aborting execution. Received \"$1\".."
	imprime "Killing child processes.."
	pgrep -P $$ | xargs kill -$1
	exitscript 1
}

SOTYPE=$(uname -s)

if [ "$SOTYPE" = "HP-UX" ]
then
	HEADCMD="head -c -n"
	ORATAB_FILE=/etc/oratab
	IDCMD=id
	AWKCMD=awk
elif [ "$SOTYPE" = "SunOS" ]
then
	HEADCMD="cut -c"
	ORATAB_FILE=/var/opt/oracle/oratab
	IDCMD=/usr/xpg4/bin/id
	AWKCMD=/usr/xpg4/bin/awk
elif [ "$SOTYPE" = "Linux" ]
then
	HEADCMD="head -c"
	ORATAB_FILE=/etc/oratab
	IDCMD=id
	AWKCMD=awk
else # ???
	imprime "S.O. \"$SOTYPE\" not yet supported.."
	exitscript 1
fi

trap "abortexp TERM" TERM

test "$EXPDEBUG" = "1" && set -x

true # This line must be the last and must stay here
####